#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"
#include <stdio.h>
#include <stdlib.h>

#define VENDOR_RQ_GET_LIGHT    0

uint16_t read_adc(uint8_t channel)
{
    ADMUX = 0b01000000; // เลือกใช้แรงดันอ้างอิงจากขา AREF
    ADMUX |= channel;   // ตั้ง MUX ให้ชี้ไปยัง channel
    ADCSRA = 0b11000110;
    //         ||   -+-
    //         ||    |
    //         ||    +-> ปรับคล็อกของ ADC ให้เป็น 1/64 ของคล็อกระบบ
    //         |+->เริ่มแปลงค่าแอนะล็อกเป็นดิจิทัล (บิต ADSC)
    //         |
    //         +-> Enable ADC (บิต ADEN)

    while ((ADCSRA & (1<<ADSC))) // รอจนบิต ADSC (บิต 6) กลายเป็น 0
        ;
    return ADCL + ADCH*256;  // ผลลัพธ์ถูกเก็บอยู่ในรีจีสเตอร์ ADCL และ ADCH
}

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t dataBuffer[1];  /* must stay valid when usbFunctionSetup returns */

    if(rq->bRequest == VENDOR_RQ_GET_LIGHT)
    {
	int shoot=0;
	srand(0);
   	uint16_t input_g = read_adc(0)+20;
	uint16_t input_v = read_adc(2)+20;
	uint16_t input_r = read_adc(3)+20;
	uint16_t input_o = read_adc(4)+20;
	uint16_t input_y = read_adc(5)+20;
    while(shoot<=20)
	{
		int count=0;
		int num;
		
		if(PORTB==0b00000000)
		{
			num=rand()%5;
		}
		while(num==0&&count==0)
		{	PORTB = 0b00000001;
			uint16_t input_g2 = read_adc(0);
			if(input_g2>input_g+10)
			{   PORTB &= 0b00000000;
			    count=1;
			    shoot+=1;
			}
		}
		while(num==1&&count==0)
		{	PORTB = 0b00000100;
			uint16_t input_v2 = read_adc(2);
  			if(input_v2>input_v+10)
        	    	{   PORTB &= 0b00000000;
			    count=1;
			    shoot+=1;
			}
		}
      	while(num==2&&count==0)
		{	PORTB = 0b00001000;
			uint16_t input_r2 = read_adc(3);
  			if(input_r2>input_r+10)
        	    	{   PORTB &= 0b00000000;
			    count=1;
			    shoot+=1; 
			}
		}
		while(num==3&&count==0)
		{	PORTB = 0b00010000;
			uint16_t input_o2 = read_adc(4);
  			if(input_o2>input_o+10)
        		    	{   PORTB &= 0b00000000;
			    count=1;
			    shoot+=1; 
			}
		}
		while(num==4&&count==0)
		{	PORTB = 0b00100000;
			uint16_t input_y2 = read_adc(5);
  			if(input_y2>input_y+10)
        	    	{   PORTB &= 0b00000000;
			    count=1;
			    shoot+=1; 
			}
		}
	}
	return 0;
	
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}


int main(void)
{
    DDRC  = 0b00000000; // PC0..PC2 = Output, PC3..PC5 = Input
    DDRB  = 0b11111111;
    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();
    sei();
    for(;;){                /* main event loop */
        usbPoll();
    }
    return 0;
}

